/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.news;

import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.util.Date;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.util.NotFoundException;
import android.content.Context;
import android.util.AttributeSet;
import android.widget.LinearLayout;
import android.webkit.WebView;
import android.widget.TextView;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.semantic.document.Document;
import it.tidalwave.bluebill.mobile.preferences.GeneralPreferences;
import it.tidalwave.bluebill.mobile.android.news.R;
import it.tidalwave.mobile.android.util.UriEncoder;
import static it.tidalwave.semantic.rss.RssVocabulary.*;

/***********************************************************************************************************************
 *
 * @stereotype View
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class AndroidNewsItemView extends LinearLayout
  {
    @Nonnull
    private final Provider<GeneralPreferences> preferences = Locator.createProviderFor(GeneralPreferences.class);    
    
    public AndroidNewsItemView (final @Nonnull Context context, final @Nonnull AttributeSet attrs) 
      {
        super(context, attrs);
      }

    public AndroidNewsItemView (final @Nonnull Context context) 
      {
        super(context);
      }
   
    public void populate (final @Nonnull PresentationModel newsItemPM) 
      {
        try 
          {
            final Document message = newsItemPM.as(Document.class);
            final String date = preferences.get().formatDateAndTime((Date)message.get(PUB_DATE));
            ((TextView)findViewById(R.id.tvDate)).setText(date);
            ((TextView)findViewById(R.id.tvTitle)).setText(message.get(TITLE));

            final String text = "<html><body>" + UriEncoder.uriEncoded(message.get(CONTENT)) + "</body></html>";
            final WebView webView = (WebView)findViewById(R.id.wvContent);
            webView.setNetworkAvailable(false);
            webView.loadData(text, "text/html", "utf-8");
          }
        catch (NotFoundException e) 
          {
            throw new RuntimeException(e);
          }
      }
  }
