/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.news;

import android.content.Context;
import android.util.AttributeSet;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.swing.Action;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.util.ui.UserNotificationWithFeedback;
import it.tidalwave.bluebill.mobile.news.ui.NewsView;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import it.tidalwave.bluebill.mobile.android.news.R;
import it.tidalwave.bluebill.mobile.news.ui.NewsViewController;
import it.tidalwave.ui.android.app.AndroidViewHelper;
import it.tidalwave.ui.android.view.AndroidBindings;
import it.tidalwave.ui.android.widget.PresentationModelAdapter;
import it.tidalwave.util.ui.UserNotification;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 *
 * @stereotype View
 * @stereotype Activity
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class AndroidNewsView extends LinearLayout implements NewsView
  {
    @CheckForNull
    private NewsViewController controller;
    
    @CheckForNull @Getter(AccessLevel.PACKAGE)
    private ListView list;

    @CheckForNull
    private Button btMarkAllMessagesAsRead;

    @CheckForNull
    private ProgressDialog progressDialog;

    @CheckForNull @Getter(AccessLevel.PACKAGE)
    private PresentationModelAdapter listAdapter;

    private final AndroidViewHelper viewHelper = new AndroidViewHelper(this);

    /*******************************************************************************************************************
     *
     * 
     ******************************************************************************************************************/
    private final OnCancelListener progressDialogCancelListener = new OnCancelListener()
      {
        public void onCancel (final @Nonnull DialogInterface dialogInterface)
          {
            controller.cancel();
          }
      };

    /*******************************************************************************************************************
     *
     * 
     ******************************************************************************************************************/
    public AndroidNewsView (final @Nonnull Context context, final @Nonnull AttributeSet attrs) 
      {
        super(context, attrs);
      }

    /*******************************************************************************************************************
     *
     * 
     ******************************************************************************************************************/
    public AndroidNewsView (final @Nonnull Context context) 
      {
        super(context);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void initialize (final @Nonnull NewsViewController controller)
      {
        log.info("initialize({})", controller);
        
        list = (ListView)findViewById(R.id.list);        
        btMarkAllMessagesAsRead = ((Button)findViewById(R.id.btMarkAllRead));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void bindActions (final @Nonnull Action markAllMessagesAsReadAction) 
      {
        post(new Runnable() 
          {
            public void run() 
              {
                log.info("bindActions({})", markAllMessagesAsReadAction);
                AndroidBindings.bind(btMarkAllMessagesAsRead, markAllMessagesAsReadAction);
              }
          });
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void lock (final @Nonnull UserNotification notification) 
      {
        post(new Runnable() 
          {
            public void run() 
              {
                log.info("lock({})", notification);
                progressDialog = ProgressDialog.show(getContext(), notification.getCaption(), notification.getText(), true, true, progressDialogCancelListener);
              }
          });
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void unlock() 
      {
        post(new Runnable() 
          {
            public void run() 
              {
                log.info("unlock()");

                if (progressDialog != null)
                  {
                    progressDialog.dismiss();
                    progressDialog = null;
                  }
              }
          });
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void notifyFeedUnavailable (final @Nonnull UserNotificationWithFeedback notification)
      {
        post(new Runnable() 
          {
            public void run() 
              {
                log.info("notifyFeedUnavailable({})", notification);
                viewHelper.showErrorDialog(notification);
              }
          });
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void populate (final @Nonnull PresentationModel rssFeedPM)
      {
        post(new Runnable()
          {
            public void run()
              {
                log.info("populate({})", rssFeedPM);
                listAdapter = new PresentationModelAdapter(getContext(), rssFeedPM); 
                AndroidBindings.bind(list, listAdapter);
              }
          });
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void notifyFeedIsCached (final @Nonnull UserNotification notification) 
      {
        log.info("notifyFeedIsCached({})", notification);
        viewHelper.showLightNotification(notification);
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void notifyAllMessagesMarkedAsRead (final @Nonnull UserNotification notification) 
      {
        log.info("notifyAllMessagesMarkedAsRead({})", notification);
        viewHelper.showLightNotification(notification);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void confirmToDownloadNews (final @Nonnull UserNotificationWithFeedback userNotificationWithFeedback) 
      {
        log.info("confirmToDownloadNews({})", userNotificationWithFeedback);
        viewHelper.showConfirmationDialog(userNotificationWithFeedback);
      }
  }
