/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.news;

import javax.annotation.Nonnull;
import it.tidalwave.util.ui.FlowController;
import it.tidalwave.semantic.document.Document;
import it.tidalwave.bluebill.mobile.news.ui.NewsViewController;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import it.tidalwave.bluebill.mobile.android.util.CommonOptionsMenuController;
import it.tidalwave.bluebill.mobile.android.util.CommonOptionsMenuControllerProvider;
import it.tidalwave.netbeans.util.Locator;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 *
 * @stereotype Activity
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class NewsActivity extends Activity
  {
    private NewsViewController controller;

    private AndroidNewsView view;
    
//    private final AndroidActivityHelper activityHelper = new AndroidActivityHelper(this);

    private final CommonOptionsMenuController commonOptionsMenuController = Locator.find(CommonOptionsMenuControllerProvider.class).createCommonOptionsMenuController(this);  
    
    private final FlowController controlFlow = new FlowController() 
      {
        public void finish() 
          {
            runOnUiThread(new Runnable() 
              {
                public void run() 
                  {
                    log.info("dismiss()");
                    NewsActivity.this.finish();       
                  }
              });
          }

        public void toNextStep (final @Nonnull Object... args) 
          {
            final Document message = (Document) args[0];
            
            try
              {
                final Intent intent = new Intent(view.getContext(), NewsItemActivity.class);
                intent.putExtra("message", message); 
                startActivity(intent);
              }
            catch (Exception e)
              {
                log.warn("Can't show news item:", e);
              }
          }  
      };

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void onCreate (final @Nonnull Bundle savedInstanceState)
      {
        log.info("onCreate()");
        super.onCreate(savedInstanceState);
        
        setContentView(R.layout.view_news);        
        view = (AndroidNewsView)findViewById(R.id.viewNews);
        controller = new AndroidNewsViewController(view, controlFlow);
        view.initialize(controller);
        registerForContextMenu(view.getList());
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void onResume() 
      {
        log.info("onResume()");
        super.onResume();
        controller.showNewsFeed();
      }    
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public boolean onCreateOptionsMenu (final @Nonnull Menu menu)
      {
        getMenuInflater().inflate(R.menu.news_options_menu, menu);
        return true;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public boolean onOptionsItemSelected (final @Nonnull MenuItem item)
      {
        return commonOptionsMenuController.onOptionsItemSelected(item);
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void onCreateContextMenu (final @Nonnull ContextMenu menu,
                                     final @Nonnull View view,
                                     final @Nonnull ContextMenuInfo menuInfo)
      {
        this.view.getListAdapter().onCreateContextMenu(menu, view, menuInfo);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean onContextItemSelected (final @Nonnull MenuItem item)
      {
        return view.getListAdapter().onContextItemSelected(item);
      }
  }
